import re
import unicodedata
from typing import Any, Dict, List

class NotVietnamese(Exception):
    pass

class SplitSyl:
    """As Vietnamese syllables are usually divided by space or hyphen,
    we will assume that the input (after the first split) has
    only one syllable.
    """
    def __init__(self, system: Dict[str, Any]) -> None:
        self.sys_dict = system

    def split_syl(self, syl) -> List[str]:
        self.syl = unicodedata.normalize('NFC', syl.lower())
        self.find_structure()
        self.find_tone()
        self.check_vowel()
        return [self.onset, self.vowel, self.coda, self.tone]

    def find_structure(self) -> None:
        pattern = f'({self.find_onset_pattern()}|{self.find_coda_pattern()})'
        expression = re.compile(pattern)
        split = re.split(expression, self.syl)
        
        self.onset = split[1]
        self.vowel = split[2]
        self.coda = split[3]

    def find_onset_pattern(self) -> str:
        onset_pattern = '|^'.join(sorted(
            self.sys_dict['onset'],
            key=lambda onset_char: len(onset_char),
            reverse=True))
        onset_pattern = f'^{onset_pattern}'
        return onset_pattern

    def find_coda_pattern(self) -> str:
        coda_pattern = '$|'.join(sorted(
            self.sys_dict['coda'],
            key=lambda coda_char: len(coda_char),
            reverse=True))
        coda_pattern = f'{coda_pattern}$'
        return coda_pattern

    def find_tone(self) -> None:
        self.tone = ''
        self.vowel = unicodedata.normalize('NFD', self.vowel)
        pattern = re.compile('|'.join(self.sys_dict['tone']))
        finding = re.search(pattern, self.vowel)
        if finding:
            self.vowel = self.vowel.replace(finding.group(0), '')
            self.tone = finding.group(0)
        self.vowel = unicodedata.normalize('NFC', self.vowel)
    
    def check_vowel(self) -> None:
        if self.vowel not in self.sys_dict['vowel']:
            raise NotVietnamese


import unittest
from wunsen import ThapSap

GENERAL = {
    # composed form
    'tiếng Việt': 'เตี๊ยง เหวียต',
    # decomposed form
    'tiếng Việt': 'เตี๊ยง เหวียต',
    'ベトナム語': 'ベトナム語',
    '越南语': '越南语',
    'ภาษาเวียดนาม': 'ภาษาเวียดนาม',
    'Từ Hán Việt詞漢越': 'ตื่อ ฮ้าน เหวียต詞漢越'
}

RS55_EXAMPLE = {
    # 'quốc ngữ': 'โกว๊กหงือ', => กว๊กหงือ
    'ngang': 'งาง',
    'ma': 'มา',
    'câm': 'เกิม',
    'cơm': 'เกิม',
    'huyền': 'ฮเหวี่ยน',
    'mà': 'หม่า',
    'bà': 'บ่า',
    'xã': 'สา',
    'rõ': 'สอ',
    'mả': 'หมา',
    'rẻ': 'แส',
    'má': 'ม้า',
    'mát': 'ม้าต',
    'mạ': 'หมะ',
    'chị': 'จิ',
    'mạn': 'หมั่น',
    'hữu': 'หืว',
    'hưu': 'ฮืว',
    'ngoan': 'งวาน',
    'ngoãn': 'งหวาน',
    'ngoen': 'แงวน',
    'ngoẻn': 'งแหวน',
    'Hội An': 'โห่ย อาน', # 'โห่ยอาน'
    'Gia Long': 'ซา ล็อง', # 'ซาล็อง'
    'Võ Nguyên Giáp': 'หวอ เงวียน ซ้าป',
    'Bà Rịa-Vũng Tàu': 'บ่า เสียะ-หวุง เต่า', # 'บ่าเสียะ-หวุงเต่า'
    'Cao Bá Quát': 'กาว บ๊า กว๊าต', # 'กาวบ๊า-กว๊าต'
    'khuây': 'เคว็ย',
    'khuấy': 'เคว้ย',

    'bay': 'บัย',
    'các': 'ก๊าก',
    'bác': 'บ๊าก',
    'cha': 'จา',
    'thích': 'ทิก',
    'da': 'ซา',
    'dậy': 'เส่ย',
    'đình': 'ดิ่ญ',
    'gà': 'ก่า',
    'ghe': 'แก',
    'già': 'ส่า',
    'giá': 'ซ้า',
    'họ': 'เหาะ',
    'ho': 'ฮอ',
    'kể': 'เก๋',
    'khi': 'คี',
    'khỉ': 'ขี',
    'lo': 'ลอ',
    'mẹ': 'แหมะ',
    'cằm': 'กั่ม',
    'no': 'นอ',
    'ăn': 'อัน',
    'ngà': 'หง่า',
    'ông': 'อง',
    'nghi': 'งี',
    'nhà': 'หญ่า',
    'sinh': 'ซิญ',
    'nhanh': 'ญัญ',
    'khép': 'แค้ป',

    'Pháp': 'ฟ้าป',
    'phở': 'เฝอ',
    'quan': 'กวาน',
    'ra': 'ซา',
    'rổ': 'โส',
    'sư': 'ซือ',
    'sả': 'สา',
    'tôi': 'โตย',
    'bút': 'บู๊ต',
    'thu': 'ทู',
    'thả': 'ถา',
    'trà': 'จ่า',
    'vui': 'วูย',
    'xa': 'ซา',
    'xã': 'สา',
    'gì': 'สี่',

    'màn': 'หม่าน',
    'nhanh': 'ญัญ',
    'ta': 'ตา',
    'mặn': 'หมั่น',
    'tân': 'เติน',
    'em': 'แอม',
    'mẹ': 'แหมะ',
    'lệnh': 'เหล่ญ',
    'bên': 'เบน',
    'tê': 'เต',
    'lịch': 'หลิก',
    'in': 'อีน',
    'đi': 'ดี',
    'Mỹ': 'หมี',
    'xong': 'ซ็อง',
    'con': 'กอน',
    'có': 'ก๊อ',
    'sông': 'ซง',
    'bốn': 'โบ๊น',
    'cô': 'โก',
    'lớn': 'เลิ้น',
    'mở': 'เหมอ',
    'chúc': 'จุ๊ก',
    'núp': 'นู้ป',
    'tủ': 'ตู๋',
    'nhưng': 'ญึง',
    'như': 'ญือ',
    'xoong': 'ซอง',
    'lôông tôông': 'โลง โตง', # 'โลงโตง'

    'ai': 'อาย',
    'bài': 'บ่าย',
    'ao': 'อาว',
    'cao': 'กาว',
    'nhau': 'เญา',
    'sáu': 'เซ้า',
    'đâu': 'เดิว',
    'cay': 'กัย',
    'cây': 'เก็ย',
    'mèo': 'แหม่ว',
    'đều': 'เด่ว',
    'bia': 'เบีย',
    'tiếng': 'เตี๊ยง',
    'dìu': 'สี่ว',
    'dịu': 'สิ่ว',
    'hoa': 'ฮวา',
    'hoặc': 'ฮหวัก',
    'khoét': 'แคว้ต',
    'nói': 'น้อย',
    'tôi': 'โตย',
    'chơi': 'เจย',
    'mua': 'มัว',
    'đưa': 'เดือ',
    'xuân': 'ซวน',
    'thuê': 'เทว',
    'vui': 'วูย',
    'gửi': 'กื๋ย',
    'buồn': 'บ่วน',
    'thuở': 'ถัว',
    'đường': 'เดื่อง',
    'hưu': 'ฮืว',
    'quý': 'กวี๊',
    'kiểu': 'เกี๋ยว',
    'yếu': 'เอี๊ยว',
    'ngoài': 'งหว่าย',
    'ngoao': 'งวาว',
    'ngoáy': 'งวั้ย',
    'khuấy': 'เคว้ย',
    'khuây': 'เคว็ย',
    'muối': 'ม้วย',
    'tuổi': 'ต๋วย',
    'nuôi': 'นวย',
    'khuya': 'เควีย',
    'duyên': 'เซวียน',
    'khuỷu': 'ขวีว',
    'tươi': 'เตือย',
    'hươu': 'เฮือว',
    'rượu': 'เสื่อว'
}

COUNTRIES = {"""Hiệp hội các quốc gia Đông Nam Á:
    - Nhà nước Brunei Darussalam
    - Vương quốc Campuchia
    - Cộng hòa Indonesia
    - Cộng hoà Dân chủ Nhân dân Lào
    - Liên bang Malaysia
    - Cộng hòa Liên bang Myanmar
    - Cộng hòa Philippines
    - Cộng hòa Singapore
    - Vương quốc Thái Lan
    - Cộng hòa Xã hội chủ nghĩa Việt Nam
    """:
    """เหียป โห่ย ก๊าก กว๊ก ซา ดง นาม อ๊า:
    - หญ่า เนื้อก Brunei Darussalam
    - เวือง กว๊ก Campuchia
    - ก่ง ฮหว่า Indonesia
    - ก่ง ฮหว่า เซิน จู๋ เญิน เซิน หล่าว
    - เลียน บาง Malaysia
    - ก่ง ฮหว่า เลียน บาง Myanmar
    - ก่ง ฮหว่า Philippines
    - ก่ง ฮหว่า Singapore
    - เวือง กว๊ก ท้าย ลาน
    - ก่ง ฮหว่า สา โห่ย จู๋ เหงีย เหวียต นาม
    """
}

class TestSpellWord(unittest.TestCase):

    def test_general(self):
        thap_sap = ThapSap('vi')
        for case in GENERAL:
            self.assertEqual(thap_sap.thap(case), GENERAL[case])

    def test_rs55_example(self):
        thap_sap = ThapSap('vi')
        for case in RS55_EXAMPLE:
            self.assertEqual(thap_sap.thap(case), RS55_EXAMPLE[case])

    def test_countries(self):
        thap_sap = ThapSap('vi')
        for case in COUNTRIES:
            self.assertEqual(thap_sap.thap(case), COUNTRIES[case])

if __name__ == '__main__':
    unittest.main()


import re
from typing import Any, Dict, List

from wunsen.splitutils.splitter import SplitSyl, NotInDict

class SplitZh(SplitSyl):
    def __init__(self, sys_dict: Dict[str, Any]) -> None:
        self.sys_dict = sys_dict

    def split_syl(self, word: str) -> List[List[str]]:
        word = word.lower()
        self.check_string(word)
        self.split_tone(word)
        self.split_rime()

        for syl in self.syls:
            self.check_rime(syl[1], syl[0])
            self.check_tone(syl[2])

        return self.syls

    @staticmethod
    def check_string(word: str) -> None:
        if not re.findall(r'[a-zü]', word):
            raise NotInDict
        else:
            return

    def split_tone(self, word: str) -> None:
        self.word = re.split(r'(\d+)', word)
        self.word = self.check_last(self.word)
        self.word = self.word[:-1]

    @staticmethod
    def check_last(syl_list: List[Any]) -> List[Any]:
        if syl_list[-1] != '':
            syl_list.extend([5, ''])
            return syl_list
        else:
            return syl_list

    def split_rime(self) -> None:

        self.syls = []
        onset_expression = re.compile(self.find_onset_pattern())

        for i, syl in enumerate(self.word):
            if i % 2 == 0:
                split_syl = re.split(onset_expression, syl)[1:]
                self.syls.extend(split_syl)
            else:
                self.syls.append(int(syl))
        
        self.syls = self.grouping(self.syls, 3)

    def find_onset_pattern(self) -> str:
        onset_pattern = '|^'.join(sorted(
            self.sys_dict['onset'],
            key=lambda chars: len(chars),
            reverse=True))
        onset_pattern = f'(^{onset_pattern})'
        return onset_pattern

    def check_rime(self, rime: str, onset: str) -> None:
        if (rime in self.sys_dict['rime']
                or (rime == '' and onset == 'r')):
            return
        else:
            raise NotInDict

    def check_tone(self, tone: str) -> None:
        if tone in self.sys_dict['tone']:
            return
        else:
            raise NotInDict


import re
from typing import Any, Dict, List, Tuple

from wunsen.splitutils.splitter import NotInDict
from .zh_splitter import SplitZh

SPLIT_EXP = re.compile(r'([A-Za-zü0-9]+)')

class AlterThird:
    def __init__(self, sys_dict: Dict[str, Any]):
        self.sys_dict = sys_dict

    def third_alter(self, text: str) -> str:
        self.original = re.split(SPLIT_EXP, text)
        self.words = [y for x, y in enumerate(self.original) if x % 2 != 0]
        self.not_words = [y for x, y in enumerate(self.original) if x % 2 == 0]

        valids, valids_tone = self.split_word()
        new_tone = self.alter_third(valids_tone)
        new_valids = self.merge_new_tone(valids, new_tone)

        return self.merge_all(new_valids)

    def split_word(self) -> Tuple[List[List[List[str]]], List[List[int]]]:
        valids = []
        valids_tone = []
        splitter = SplitZh(self.sys_dict)
        splitter.sys_dict = self.sys_dict
        for word in self.words:
            try:
                split = splitter.split_syl(word)
                if split:
                    valids.append(split)
                    valids_tone.append([x[2] for x in split])
            except NotInDict:
                valids.append([])
                valids_tone.append([])
        return valids, valids_tone

    def alter_third(self, valids_tone: List[List[int]]) -> List[List[int]]:
        reversed_tone = self.tone_reverse(valids_tone)
        new_reversed_tone = []
        for index, word_group in enumerate(reversed_tone):
            if not word_group:
                new_reversed_tone.append(word_group)
            else:
                if index == 0:
                    third = False
                else:
                    third = self.check_third(new_reversed_tone[-1])
                new_syl, third = self.alter_inner_third(word_group, third)
                new_reversed_tone.append(new_syl)
        new_tone = self.tone_reverse(new_reversed_tone)
        return new_tone

    @staticmethod
    def alter_inner_third(
            word_group: List[int],
            third: bool) -> Tuple[List[int], bool]:
        new_syl = []
        for syl in word_group:
            tone = syl
            if syl == 3:
                if third == True:
                    tone = 2
                third = True
            else:
                third = False
            new_syl.append(tone)
        return new_syl, third

    @staticmethod
    def tone_reverse(tones: List[List[int]]) -> List[List[int]]:
        tones = [list(reversed(sublist)) for sublist in reversed(tones)]
        return tones

    @staticmethod
    def check_third(tones: List[int]) -> bool:
        return tones and tones[-1] == 3

    @staticmethod
    def merge_new_tone(
            valids: List[List[List[str]]],
            new_tone: List[List[int]]) -> List[List[List[str]]]:
        new_valids = []
        for index, valid in enumerate(valids):
            new_valid = []
            if valid:
                for syl_index, syl in enumerate(valid):
                    new_syl = syl
                    new_syl[2] = new_tone[index][syl_index]
                    new_valid.append(new_syl)
            new_valids.append(new_valid)
        return new_valids

    def merge_all(self, new_valids: List[List[List[str]]]) -> str:
        new_text = [self.not_words[0]]
        for index, valid in enumerate(new_valids):
            if not valid:
                new_text.append(self.words[index])
            else:
                new_valid = []
                for syl in valid:
                    new_valid.append(''.join([str(char) for char in syl]))
                new_text.append(''.join(new_valid))
            new_text.append(self.not_words[index+1])
        new_text = ''.join(new_text)
        return new_text


import re
from typing import Any, Dict, List

from khanaa import SpellWord

from wunsen.splitutils.splitter import NotInDict
from wunsen.splitutils.exception import NotAvailableSystem
from .zh_splitter import SplitZh
from .zh_mapping import RI49, THC43
from .zh_sandhi import AlterThird, SPLIT_EXP

class ZhRi49:

    zh_dict = RI49
    short_forth_list = ['ai', 'an', 'ang', 'ao', 'uai', 'uang']
    aao_aang = {
        'w': ['u'],
        'y': ['i']
    }

    def __init__(self) -> None:
        pass

    def thap_kham(self, word: str) -> str:
        try:
            splitter = SplitZh(self.zh_dict)
            splitter.sys_dict = self.zh_dict
            self.split = splitter.split_syl(word)
        except NotInDict:
            return word
        result_list = []
        self.pref = {}
        spell = SpellWord(**{
            'clear_vowel': False,
            'obvious_h_low_single': False})
        for index, syl in enumerate(self.split):
            if self.is_erhua(syl):
                if len(self.split) == 1:
                    return word
                else:
                    continue
            self.is_erhua_next(index)
            self.shorten = False
            pattern = self.turn_thai(syl)
            self.select_pref(syl)
            spell.option.update(**self.pref)
            result_list.append(spell.spell_out(**pattern))

        result = ''.join(result_list)

        return result

    @staticmethod
    def is_erhua(syl: List) -> bool:
        return syl[0] == 'r' and syl[1] == ''
    
    def is_erhua_next(self, index) -> None:
        self.erhua_next = False
        if (index < len(self.split) - 1
                and self.is_erhua(self.split[index+1])):
            self.erhua_next = True

    def turn_thai(self, syl: List) -> Dict[str, Any]:
        self.select_rime(syl)
        self.select_onset(syl)
        self.select_tone(syl)
        self.short_fifth(syl)
        pattern = {
            'onset': self.onset,
            'vowel': self.rime[1],
            'silent_before': self.rime[2],
            'coda': self.rime[3],
            'tone': self.tone
        }
        if self.erhua_next:
            pattern = self.add_erhua(pattern)
        return pattern

    def select_rime(self, syl: List) -> None:
        self.rime = self.zh_dict['rime'][syl[1]]

        self.short_fourth_a(syl)

        self.rime_dict = 'rime'
        if (syl[1] in self.zh_dict['cond_rime']
                and syl[0] in self.zh_dict['cond_rime'][syl[1]]):
            self.rime = self.zh_dict['rime_cond'][syl[1]]
            self.rime_dict = 'rime_cond'

    def short_fourth_a(self, syl: List) -> None:
        if (syl[1] in self.short_forth_list
                and syl[2] == 4):
            self.shorten = True

    def select_onset(self, syl: List) -> None:
        self.onset = self.zh_dict['onset'][syl[0]]
        vowel_onset = self.zh_dict[self.rime_dict][syl[1]][0]

        if (syl[0] in self.aao_aang
                and syl[1] in self.aao_aang[syl[0]]):
            self.onset = 'อ'
        if vowel_onset != 'อ':
            if syl[0] == '':
                self.onset = vowel_onset
            else:
                self.onset = ''.join([self.onset, vowel_onset])

    def select_tone(self, syl: List) -> None:
        self.tone = self.zh_dict['tone'][syl[2]]

    def short_fifth(self, syl: List) -> None:
        if (syl[2] in [0, 5]
                and (syl[1] == 'a'
                or (syl[1] == 'i'
                and syl[0] in self.zh_dict['cond_rime'][syl[1]]))):
            self.shorten = True
            self.tone = 1

    @staticmethod
    def add_erhua(pattern: Dict[str, Any]) -> Dict[str, Any]:
        return pattern

    def select_pref(self, syl: List) -> None:
        if self.shorten == True:
            self.pref.update({'vowel_length': 'short'})
        else:
            self.pref.update({'vowel_length': 'input'})
        if (self.rime[4]
                or (self.onset == 'ฮว' and self.tone in [1, 4])):
            self.pref.update({'onset_style': 'phinthu'})
        else:
            self.pref.update({'onset_style': 'plain'})

class ZhThc43(ZhRi49):
    
    zh_dict = THC43
    short_forth_list = ['an']
    aao_aang = {
        'w': ['u'],
        'y': ['i', 'in', 'ing', 'u', 'un', 'v', 'vn']
    }

    def short_fifth(self, syl: List) -> None:
        pass

    @staticmethod
    def add_erhua(pattern: Dict[str, Any]) -> Dict[str, Any]:
        if pattern['silent_before']:
            new_silent_before = ''.join([
                pattern['silent_before'], 'ร'])
            pattern['silent_before'] = new_silent_before
        else:
            pattern.update({'silent_after': 'ร'})
        return pattern

    def select_pref(self, syl: List) -> None:
        super().select_pref(syl)
        if syl[0] == 'w' and syl[1] == 'an':
            self.pref.update({'vowel_length': 'short'})
        elif syl[1] == 'ei' and not self.erhua_next:
            self.pref.update({'silent_before_style': 'plain'})

class ThapZh:
    def __init__(
        self, system: str = 'THC43',
        input: str = 'Pinyin-number',
        option: Dict[str, Any] = {'sandhi': True}) -> None:
        """Setting
        
        :param system: Select thapsap system.
            - 'RI49' for the Royal Institute (2006/2549) system
            - 'THC43' for เกณฑ์การถ่ายทอดเสียงภาษาจีนแมนดาริน
            ด้วยอักขรวิธีไทย (2000/2543)

        :param input: Select input type.
            - 'Pinyin-number' for Hanyu Pinyin with tone number
            (ex. xie4xie5)
        
        :param option: Select option
            - for third tone sandhi
                - {'sandhi': True} for automatic third tone sandhi
                (ex. ni3 hao3 => หนีห่าว (ni2 hao3))
                - {'sandhi': False}
                (ex. ni3 hao3 => หนี่ห่าว (ni3 hao3))
        """
        if system == 'RI49' and input == 'Pinyin-number':
            self.transcript = ZhRi49()
            self.sys_dict = RI49
        elif system == 'THC43' and input == 'Pinyin-number':
            self.transcript = ZhThc43()
            self.sys_dict = THC43
        else:
            raise NotAvailableSystem
        self.option = option

    def thap(self, text: str) -> str:
        if self.option['sandhi']:
            alter_third = AlterThird(self.sys_dict)
            text = alter_third.third_alter(text)
        def replace(match):
            return self.transcript.thap_kham(match.group())
        return re.sub(SPLIT_EXP, replace, text)


import unittest
from wunsen import ThapSap

SANDHI_ON = {
    'ni3 hao3': 'หนี ห่าว',
    'ni3hao3': 'หนีห่าว',
    'bao3guan3 hao3': 'ป๋าวกวั๋น ห่าว',
    'lao3 bao3guan3': 'หล่าว ป๋าวกวั่น'
}

SANDHI_OFF = {
    'ni3 hao3': 'หนี่ ห่าว',
    'ni3hao3': 'หนี่ห่าว',
    'bao3guan3 hao3': 'ป่าวกวั่น ห่าว',
    'lao3 bao3guan3': 'หล่าว ป่าวกวั่น'
}

RI49_GENERAL = {
    'กขคงzhong1': 'กขคงจง',
    'zai4jian4': 'ไจ้เจี้ยน',
    '[[zai4jian4]]': '[[ไจ้เจี้ยน]]',
    '55 a1 a23 a4 a5 a6 abcd1 1': '55 อา a23 อ้า อะ a6 abcd1 1',
    'ben3r5': 'เปิ่น'
}

RI49_EXAMPLE = {
    'ba1': 'ปา',
    'can2': 'ฉาน',
    'cang1': 'ชาง',
    'cha2': 'ฉา',
    'chi1': 'ชือ',
    'da4': 'ต้า',
    'fen2': 'เฝิน',
    'fan4': 'ฟั่น',
    'ge1': 'เกอ',
    'hao3': 'ห่าว',
    'he1': 'เฮอ',
    'jiang1': 'เจียง',
    'kui2': 'ขุย',
    'kan4': 'คั่น',
    'lai2': 'หลาย',
    'ma1': 'มา',
    'nan2': 'หนาน',
    'pa2': 'ผา',
    'po1': 'พัว',
    'qian2': 'เฉียน',
    'qiang1': 'เชียง',
    'ren2': 'เหริน',
    'san1': 'ซาน',
    'sao3': 'ส่าว',
    'sheng2': 'เฉิง',
    'shu1': 'ชู',
    'Tang2': 'ถาง',
    'ta1': 'ทา',
    'wan4': 'วั่น',
    'wo3': 'หวั่ว',
    'wu3': 'อู่',
    'xiang1': 'เซียง',
    'xie2': 'เสีย',
    'yin2': 'หยิน',
    'yi1': 'อี',
    'zou3': 'โจ่ว',
    'zhan4': 'จั้น',
    
    'ma5': 'หมะ',
    'ma1': 'มา',
    'tai4': 'ไท่',
    'mai3': 'หม่าย',
    'kan4': 'คั่น',
    'lan2': 'หลาน',
    'chang4': 'ชั่ง',
    'chang2': 'ฉาง',
    'bao4': 'เป้า',
    'gao1': 'กาว',
    'le4': 'เล่อ',
    'ye4': 'เย่',
    # 'ê2': 'เอ๋',
    'fei1': 'เฟย์',
    'ben3': 'เปิ่น',
    'deng3': 'เติ่ง',
    'er4': 'เอ้อร์',
    'ni3': 'หนี่',
    'er2zi5': 'เอ๋อร์จึ',
    'shi3': 'ฉื่อ',
    'jia1': 'เจีย',
    'tian1': 'เทียน',
    'niang2': 'เหนียง',
    'biao3': 'เปี่ยว',
    'jie4': 'เจี้ย',
    'lin2': 'หลิน',
    'ding1': 'ติง',
    'xiong1': 'ซฺยง',
    'xiong2': 'สฺยง',
    'qiong2': 'ฉฺยง',
    'jiong3': 'จฺย่ง',
    'niu2': 'หนิว',
    'mo1': 'มัว',
    'o1yo1': 'โอโย',
    'long2': 'หลง',
    'lou2': 'โหลว',
    'bu4': 'ปู้',
    'yu2': 'ยฺหวี',
    'nü3': 'นฺหวี่',
    'gua1': 'กวา',
    'guai1': 'กวาย',
    'duan4': 'ตวั้น',
    'quan4': 'เชฺวี่ยน',
    'guang1': 'กวาง',
    'yue4': 'เยฺว่',
    'nüe4': 'เนฺว่',
    'hui2': 'หุย',
    'dun4': 'ตุ้น',
    'yun2': 'ยฺหวิน',
    'guo2': 'กั๋ว'
}

RI49_COUNTRIES = {"""wen2lai2
jian3pu3zhai4
yin4ni2
lao3wo1
ma3lai2xi1ya4
mian3dian4
fei1lü4bin1
xin1jia1po1
tai4guo2
yue4nan2""":
"""เหวินหลาย
เจี๋ยนผู่ไจ้
ยิ่นหนี
หล่าววัว
หม่าหลายซีย่า
เหมี่ยนเตี้ยน
เฟย์ลฺวี่ปิน
ซินเจียพัว
ไท่กั๋ว
เยฺว่หนาน"""
}

THC43_GENERAL = {
    'กขคงzhong1': 'กขคงจง',
    'zai4jian4': 'ไจ้เจี้ยน',
    '[[zai4jian4]]': '[[ไจ้เจี้ยน]]',
    '55 a1 a23 a4 a5 a6 abcd1 1': '55 อา a23 อ้า อา a6 abcd1 1',
    'ben3r5': 'เปิ่นร์'
}

THC43_EXAMPLE = {
    'ba1': 'ปา',
    'pa1': 'พา',
    'ma1': 'มา',
    'fa1': 'ฟา',
    'da1': 'ตา',
    'ta1': 'ทา',
    'na1': 'นา',
    'la1': 'ลา',
    'zi1': 'จือ',
    'ci1': 'ชือ',
    'si1': 'ซือ',
    'zhi1': 'จือ',
    'chi1': 'ชือ',
    'shi1': 'ซือ',
    'ji1': 'จี',
    'qi1': 'ชี',
    'xi1': 'ซี',
    'ri4': 'รื่อ',
    'ga1': 'กา',
    'ka1': 'คา',
    'ha1': 'ฮา',

    'ba4': 'ป้า',
    'bo4': 'ปั้ว',
    'bai4': 'ไป้',
    'bei4': 'เป้ย',
    'bao4': 'เป้า',
    'pou3': 'โผ่ว',
    'ban4': 'ปั้น',
    'ben4': 'เปิ้น',
    'bang4': 'ปั้ง',
    'beng4': 'เปิ้ง',
    'bi4': 'ปี้',
    'biao4': 'เปี้ยว',
    'bie4': 'เปี้ย',
    'miu4': 'มิ่ว',
    'bian4': 'เปี้ยน',
    'bin4': 'ปิ้น',
    'bing4': 'ปิ้ง',
    'bu4': 'ปู้',

    'de2': 'เต๋อ',
    'dong1': 'ตง',
    'duo1': 'ตัว',
    'dui1': 'ตุย',
    'duan1': 'ตวน',
    'dun1': 'ตุน',

    'jia3': 'จย่า',
    'jiang3': 'เจี่ยง',
    'jiong3': 'จย่ง',
    'zhu1': 'จู',
    'zhua1': 'จวา',
    'zhuo1': 'จัว',
    'zhuai1': 'ไจว',
    'zhui1': 'จุย',
    'zhuan1': 'จวน',
    'zhun1': 'จุน',
    'zhuang1': 'จวง',
    'ju1': 'จีว์',
    'jue1': 'เจวีย',
    'juan1': 'เจวียน',
    'jun1': 'จวิน',
    
    'a1': 'อา',
    'e1': 'เออ',
    'er2': 'เอ๋อร์',
    'ai1': 'ไอ',
    'ao1': 'เอา',
    'ou1': 'โอว',
    'an1': 'อาน',
    'en1': 'เอิน',
    'ang1': 'อัง',
    'yi2': 'อี๋',
    'ya2': 'หยา',
    'yao2': 'เหยา',
    'ye2': 'เหยีย',
    'you2': 'โหยว',
    'yan2': 'เหยียน',
    'yin2': 'อิ๋น',
    'yang2': 'หยัง',
    'ying2': 'อิ๋ง',
    'yong2': 'หยง',
    'wu4': 'อู้',
    'wa4': 'ว่า',
    'wo4': 'วั่ว',
    'wai4': 'ไว่',
    'wei4': 'เว่ย',
    'wan4': 'วั่น',
    'wen4': 'เวิ่น',
    'wang4': 'วั่ง',
    'weng4': 'เวิ่ง',
    'yu4': 'อี้ว์',
    'yue4': 'เย่ว์',
    'yuan4': 'ย่วน',
    'yun4': 'อวิ้น'
}

class TestSpellWord(unittest.TestCase):

    def test_sandhi_on(self):
        thap_sap = ThapSap('zh', system='RI49')
        for case in SANDHI_ON:
            self.assertEqual(thap_sap.thap(case), SANDHI_ON[case])

    def test_sandhi_off(self):
        thap_sap = ThapSap('zh', system='RI49', option={'sandhi': False})
        for case in SANDHI_OFF:
            self.assertEqual(thap_sap.thap(case), SANDHI_OFF[case])

    def test_ri49_general(self):
        thap_sap = ThapSap('zh', system='RI49')
        for case in RI49_GENERAL:
            self.assertEqual(thap_sap.thap(case), RI49_GENERAL[case])

    def test_ri49_example(self):
        thap_sap = ThapSap('zh', system='RI49')
        for case in RI49_EXAMPLE:
            self.assertEqual(thap_sap.thap(case), RI49_EXAMPLE[case])

    def test_countries(self):
        thap_sap = ThapSap('zh', system='RI49')
        for case in RI49_COUNTRIES:
            self.assertEqual(thap_sap.thap(case), RI49_COUNTRIES[case])

    def test_thc43_general(self):
        thap_sap = ThapSap('zh', system='THC43')
        for case in THC43_GENERAL:
            self.assertEqual(thap_sap.thap(case), THC43_GENERAL[case])

    def test_thc43_example(self):
        thap_sap = ThapSap('zh', system='THC43')
        for case in THC43_EXAMPLE:
            self.assertEqual(thap_sap.thap(case), THC43_EXAMPLE[case])

if __name__ == '__main__':
    unittest.main()


from typing import Any

from .vietnamese import ThapVi
from .japanese import ThapJa
from .korean import ThapKo
from .mandarin import ThapZh

class ThapSap:
    def __init__(self, lang: str, **setting: Any) -> None:
        """Setting
        
        :param lang: Specify language to use ('ja' Japanese |
                    'ko' Korean | 'vi' Vietnamese |
                    'zh' Standard Chinese)
        
        The rest are not required.
        
        :key str system: Specify system of transcription/
                    transliteration to use. Available system:
                    ja: 'ORS61', 'RI35' | ko: 'RI55' | vi: 'RI55' |
                    zh: 'RI49', 'THC43'
        :key str input: Specify input type
                    ja: 'Hepburn-macron', 'Hepburn-no diacritic' |
                    ko: 'RR' | vi: 'VA' | zh: 'Pinyin-number'
        :key dict option: (Standard Chinese only) Specify option
                    zh: {'sandhi': True/False} (for third tone sandhi)
        """
        if lang == 'ja':
            self._transcriber = ThapJa(**setting)
        elif lang == 'ko':
            self._transcriber = ThapKo(**setting)
        elif lang == 'vi':
            self._transcriber = ThapVi(**setting)
        elif lang == 'zh':
            self._transcriber = ThapZh(**setting)
        else:
            raise ValueError('Language not found')

    def thap(self, text: str) -> str:
        return self._transcriber.thap(text)


import unittest
from wunsen import ThapSap

GENERAL = {
    "한국말, han'gugeo, कोरियाई भाषा": '한국말, ฮันกูกอ, कोरियाई भाषा',
    'Koreanische Sprache': 'Koreanische Sprache'
}

RI55_EXAMPLE = {
    "baji": "พาจี",
    "haengbok": "แฮ็งบก",
    "bap": "พับ",
    "cheonji": "ช็อนจี",
    "Chungcheong": "ชุงช็อง",
    "kkot": "กด",
    "Daegu": "แทกู",
    "daedap": "แทดับ",
    "natgari": "นัดการี",
    # "Gangwon": "คังว็อน",
    "Hanguk": "ฮันกุก",
    "goguk": "โคกุก",
    "hobak": "โฮบัก",
    "gonghang": "คงฮัง",
    "jido": "ชีโด",
    "jujeonja": "ชูจ็อนจา",
    "bit": "พิด",
    "jjari": "จารี",
    "gajja": "คาจา",
    "kal": "คัล",
    "mankeum": "มันคึม",
    "bueok": "พูอ็อก",
    "chukha": "ชูคา",
    "kkangtong": "กังทง",
    "eokkae": "ออแก",
    "bak": "พัก",
    # "maeum": "มาอึม",
    "nonmun": "นนมุน",
    "gim": "คิม",
    "na": "นา",
    "nuna": "นูนา",
    "Namdaemun": "นัมแดมุน",
    "Gangneung": "คังนึง",
    "madang": "มาดัง",
    "pal": "พัล",
    "apeuda": "อาพือดา",
    "yeopsaram": "ย็อบซารัม",
    "iphak": "อีพัก",
    "ppallae": "ปัลแล",
    "ippal": "อีปัล",
    "ramyeon": "รามย็อน",
    "haru": "ฮารู",
    "mal": "มัล",
    "Jeolla": "ช็อลลา",
    "sada": "ซาดา",
    "saengsan": "แซ็งซัน",
    "sutso": "ซุดโซ",
    "sijang": "ชีจัง",
    "dosi": "โทชี",
    "ssal": "ซัล",
    "bulssuk": "พุลซุก",
    # "ssaetta": "แซ็ดตา",
    "Taebaek": "แทแบ็ก",
    "gita": "คีทา",
    "Hanbat": "ฮันบัด",
    "mathyeong": "มาทย็อง",
    "ttukkeong": "ตูก็อง",
    "heoritti": "ฮอรีตี",

    "ai": "อาอี",
    "nara": "นารา",
    "ap": "อับ",
    "bap": "พับ",
    "aein": "แออิน",
    "gae": "แค",
    "aengmusae": "แอ็งมูแซ",
    "naemsae": "แน็มแซ",
    "e": "เอ",
    "nemo": "เนโม",
    "enganhada": "เอ็นกันฮาดา",
    "sem": "เซ็ม",
    "eomeoni": "ออมอนี",
    "seoda": "ซอดา",
    "eonni": "อ็อนนี",
    "deoreopta": "ทอร็อบทา",
    "euro": "อือโร",
    "seuseuro": "ซือซือโร",
    # "maeum": "มาอึม",
    "ireum": "อีรึม",
    "ima": "อีมา",
    "si": "ชี",
    # "Yongin": "ยงอิน",
    "sil": "ชิล",
    "o": "โอ",
    "podo": "โพโด",
    "ot": "อด",
    "don": "ทน",
    "ugi": "อูกี",
    "dubu": "ทูบู",
    "undong": "อุนดง",
    "sul": "ซุล",

    "oeguk": "เวกุก",
    "goeroum": "คเวโรอุม",
    "oenson": "เว็นซน",
    "hoengdan": "ฮเว็งดัน",
    "uisa": "อึยซา",
    "Yeouido": "ยออีโด",
    "huin saek": "ฮีน แซ็ก", # "ฮีนแซ็ก"
    "wa": "วา",
    "gwaja": "ควาจา",
    "wang": "วัง",
    "Gwangju": "ควังจู",
    "wae": "แว",
    "dwaeji": "ทแวจี",
    # "waengwaeng": "แว็งแว็ง",
    "kkwaengnamu": "กแว็งนามู",
    "weiteo": "เวอีทอ",
    "gwebeom": "คเวบ็อม",
    "wennil": "เว็นนิล",
    "wi": "วี",
    "gwi": "ควี",
    "witsaram": "วิดซารัม",
    "swin": "ชวิน",
    "areumdawo!": "อารึมดาวอ!", # "อารึมดาวอ"
    "jwo": "ชวอ",
    "wollae": "ว็อลแล",
    "yeogwon": "ยอกว็อน",
    "yagu": "ยากู",
    "chyawo": "ชยาวอ",
    "yak": "ยัก",
    "dalgyal": "ทัลกยัล",
    "yaegi": "แยกี",
    "gyae": "คแย",
    "yejeol": "เยจ็อล",
    "sigye": "ชีกเย",
    "yennal": "เย็นนัล",
    "gyetnal": "คเย็ดนัล",
    "yeohaeng": "ยอแฮ็ง",
    "byeo": "พยอ",
    "yeonmal": "ย็อนมัล",
    "ramyeon": "รามย็อน",
    "yori": "โยรี",
    "hakgyo": "ฮักกโย",
    "yong": "ยง",
    "gongryong": "คงรยง",
    "uyu": "อูยู",
    "hyuji": "ฮยูจี",
    "yuk": "ยุก",
    "gyul": "คยุล"
}

COUNTRIES = {
    """beurunai
kambodia
indonesia
raoseu
malleisia
miyanma
pillipin
singgaporeu
taeguk
beteunam

dongtimoreu, papuanyugini

jungguk, ilbon, daehanmin'guk
oseuteureillia, indo, nyujillaendeu, reosia, miguk""":
"""พือรูนาอี
คัมโบดีอา
อินโดเนชีอา
ราโอซือ
มัลเลอีชีอา
มียันมา
พิลลีพิน
ชิงกาโพรือ
แทกุก
เพทือนัม

ทงทีโมรือ, พาพูอานยูกีนี

ชุงกุก, อิลบน, แทฮันมินกุก
โอซือทือเรอิลลีอา, อินโด, นยูจิลแล็นดือ, รอชีอา, มีกุก"""
}

class TestSpellWord(unittest.TestCase):

    def test_general(self):
        thap_sap = ThapSap('ko')
        for case in GENERAL:
            self.assertEqual(thap_sap.thap(case), GENERAL[case])

    def test_ri55_example(self):
        thap_sap = ThapSap('ko')
        for case in RI55_EXAMPLE:
            self.assertEqual(thap_sap.thap(case), RI55_EXAMPLE[case])

    def test_countries(self):
        thap_sap = ThapSap('ko')
        for case in COUNTRIES:
            self.assertEqual(thap_sap.thap(case), COUNTRIES[case])

if __name__ == '__main__':
    unittest.main()


import re
from typing import List

from khanaa import SpellWord

from wunsen.splitutils.exception import NotAvailableSystem
from wunsen.splitutils.splitter import SplitSyl, NotInDict
from .ko_mapping import RI55

class KoRi55:

    ko_dict = RI55

    def __init__(self) -> None:
        pass

    def thap_kham(self, word: str) -> str:
        try:
            splitter = SplitSyl(self.ko_dict)
            self.split = splitter.split_syl(word)
        except NotInDict:
            return word
        thai_transcript = []
        for index, syl in enumerate(self.split):
            self.pref = {'clear_vowel_onset': 'all'}
            syl_info = {
                'onset': self.select_onset(index, syl),
                'vowel': self.select_vowel(index, syl),
                'coda': self.select_coda(syl)
            }
            spell = SpellWord(**self.pref)
            thai_transcript.append(spell.spell_out(**syl_info))
        return ''.join(thai_transcript)

    def select_onset(self, index: int, syl: List[str]) -> str:
        vowel_onset = self.ko_dict['vowel'][syl[1]][0]
        if syl[0] == '':
            return vowel_onset
        elif syl[0] == 's' and syl[1] in ['i', 'wi']:
            onset = 'ช'
        else:
            if index == 0:
                onset = self.ko_dict['onset'][syl[0]][0]
            else:
                onset = self.ko_dict['onset'][syl[0]][1]
        if vowel_onset != 'อ':
            onset = ''.join([onset, vowel_onset])
        return onset
    
    def select_vowel(self, index: int, syl: List[str]) -> str:
        if syl[1] == 'ui':
            if syl[0] == '' and index == 0:
                vowel = self.ko_dict['vowel'][syl[1]][1]
            else:
                vowel = 'อี'
        else:
            vowel = self.ko_dict['vowel'][syl[1]][1]
            if syl[2] != '':
                self.pref.update({'vowel_length': 'short'})
        return vowel

    def select_coda(self, syl: List[str]) -> str:
        if syl[2] == '':
            coda = ''
        else:
            coda = self.ko_dict['coda'][syl[2]]
        return coda

class ThapKo:
    def __init__(self, system: str = 'RI55', input: str = 'RR') -> None:
        """Setting
        
        :param system: Select thapsap system.
            - 'RI55' for the Royal Institute (2012/2555) system

        :param input: Select input type.
            - 'RR' for Revised Romanization
        """
        if system == 'RI55' and input == 'RR':
            self.transcript = KoRi55()
        else:
            raise NotAvailableSystem

    def thap(self, text: str) -> str:
        def replace(match):
            return self.transcript.thap_kham(match.group())
        return re.sub(
            r"([a-zA-Zāēīōū]|(?<=[a-zA-Zāēīōū])'(?=[a-zA-Zāēīōū]))+",
            replace, text)


import re
from typing import List, Pattern
import unicodedata

from khanaa import SpellWord

from wunsen.splitutils.splitter import SplitSyl, NotInDict
from wunsen.splitutils.exception import NotAvailableSystem
from .ja_mapping import ORS61, ORS61_NO_DIACRITIC, RI35, RI35_NO_DIACRITIC

class JaOrs61:

    coda_ng = ['g', 'k', 'h', 'f', 'w', 'y']
    coda_m = ['b', 'm', 'p']

    def __init__(self, input: str) -> None:
        self.ja_dict = ORS61
        if input == 'Hepburn-no diacritic':
            self.ja_dict = ORS61_NO_DIACRITIC

    def thap_kham(self, word: str) -> str:
        try:
            splitter = SplitSyl(self.ja_dict)
            word = unicodedata.normalize('NFC', word)
            self.split = splitter.split_syl(word)
        except NotInDict:
            return word
        self.last_syl_index = len(self.split) - 1
        self.split = self.adapt_split(self.split, self.last_syl_index)
        thai_transcript = []
        for index, syl in enumerate(self.split):
            self.pref = {}
            syl_info = {
                'onset': self.select_onset(index, syl),
                'vowel': self.select_vowel(index, syl),
                'coda': self.select_coda(index, syl)
                }
            spell = SpellWord(**self.pref)
            thai_transcript.append(spell.spell_out(**syl_info))
        return ''.join(thai_transcript)

    @staticmethod
    def adapt_split(
            old_syl: List[List[str]],
            last_syl_index: int) -> List[List[str]]:
        new_split = []
        for index, syl in enumerate(old_syl):
            new_syl = syl
            # ya = ยา not เอีย, เยีย
            if new_syl[0] == '' and new_syl[1][0] == 'y':
                new_syl = ['y', new_syl[1][1:], new_syl[2]]
            # mitsu = มิตสึ
            if (index != last_syl_index
                    and new_syl[2] == ''
                    and old_syl[index+1][0] == 'ts'):
                new_syl[2] = 't'
            # no coda for these vowels
            if (new_syl[1] in ['yo', 'yoo', 'you', 'yu', 'yuu', 'yō', 'yū']
                    and new_syl[2] != ''):
                new_syl[2] = ''
            new_split.append(new_syl)
        return new_split

    def select_onset(self, index: int, syl: List[str]) -> str:
        if syl[0] == '':
            onset = 'อ'
        elif syl[0] == 'ts' and syl[1] == 'u':
            onset = 'ส'
        else:
            if index == 0:
                onset = self.ja_dict['onset'][syl[0]][0]
            else:
                onset = self.ja_dict['onset'][syl[0]][1]
        return onset

    def select_vowel(self, index: int, syl: List[str]) -> str:
        if syl[0] == 'ts' and syl[1] in ['u', 'uu', 'ū']:
            if syl[1] == 'u':
                vowel = 'อึ'
            elif syl[1] in ['uu', 'ū']:
                vowel = 'อือ'
        else:
            vowel = self.ja_dict['vowel'][syl[1]]
            self.shorten_vowel(index, syl)
        return vowel
    
    def shorten_vowel(self, index: int, syl: List[str]) -> None:
        if (syl[1] in ['a', 'e', 'o', 'u']
                and syl[2] == ''
                and index != self.last_syl_index):
            self.pref.update({'vowel_length': 'long'})

    def select_coda(self, index: int, syl: List[str]) -> str:
        if syl[2] == '':
            coda = ''
        elif syl[2] == 's':
            coda = self.select_coda_s(index, syl)
        elif syl[2] == 'n':
            coda = self.select_coda_n(index, syl)
        else:
            coda = self.ja_dict['coda'][syl[2]]
        return coda

    def select_coda_s(self, index: int, syl: List[str]) -> str:
        if (index != self.last_syl_index
                and self.split[index+1][0] == 'sh'):
            coda = 'ช'
        else:
            coda = self.ja_dict['coda'][syl[2]]
        return coda

    def select_coda_n(self, index: int, syl: List[str]) -> str:
        if index == self.last_syl_index:
            coda = 'ง'
            return coda
        next_onset = self.split[index+1][0]
        if next_onset == '':
            coda = 'ง'
        elif next_onset in self.coda_ng:
            coda = 'ง'
        elif next_onset in self.coda_m:
            coda = 'ม'
        else:
            coda = self.ja_dict['coda'][syl[2]]
        return coda

class JaRi35(JaOrs61):

    coda_ng = ['g', 'k', 'h', 'f', 'w']

    def __init__(self, input: str) -> None:
        self.ja_dict = RI35
        if input == 'Hepburn-no diacritic':
            self.ja_dict = RI35_NO_DIACRITIC

    def shorten_vowel(self, index: int, syl: List[str]) -> None:
        pass

    def select_coda_s(self, index: int, syl: List[str]) -> str:
        coda = self.ja_dict['coda'][syl[2]]
        return coda

class ThapJa:

    def __init__(
            self, system: str = 'ORS61',
            input: str = 'Hepburn-macron') -> None:
        """Setting
        
        :param system: Select thapsap system.
            - 'ORS61' for the Office of the Royal Society (2018/2561)
            system
            - 'RI35' for the Royal Institute (1992/2535) system

        :param input: Select input type.
            - 'Hepburn-macron' for Hepburn romanization with macron
            (ex. arigatō)
            - 'Hepburn-no diacritic' for Hepburn romanization
            without diacritic (ex. arigatou)
        """
        if system == 'ORS61':
            self.transcript = JaOrs61(input)
        elif system == 'RI35':
            self.transcript = JaRi35(input)
        else:
            raise NotAvailableSystem

    def thap(self, text: str) -> str:
        def find_expression() -> Pattern[str]:
            char_list = ''.join(['a-zA-ZāēīōūĀĒĪŌŪ', u'\u00af'])
            exp = f"([{char_list}]|(?<=[{char_list}])'(?=[{char_list}]))+"
            return re.compile(exp)
        def replace(match):
            return self.transcript.thap_kham(match.group())
        return re.sub(find_expression(), replace, text)


import re

from khanaa import SpellWord

from .vi_mapping import RI55
from .vi_splitter import NotVietnamese, SplitSyl

_VI_ALPHABET = (r'a-zA-Zàáãạảăắằẳẵặâấầẩẫậèéẹẻẽêềếểễệđìíĩỉịòóõọỏôốồổỗộơớờởỡợ'
    r'ùúũụủưứừửữựỳỵỷỹýÀÁÃẠẢĂẮẰẲẴẶÂẤẦẨẪẬÈÉẸẺẼÊỀẾỂỄỆĐÌÍĨỈỊÒÓÕỌỎÔỐỒỔỖỘƠỚỜỞỠỢ'
    r'ÙÚŨỤỦƯỨỪỬỮỰỲỴỶỸÝ')

_VI_DIACRITIC = ''.join([u'\u0300', u'\u0303', u'\u0309', u'\u0301',
    u'\u0323', u'\u0302'])

_VI_ALL = ''.join(f'[{_VI_ALPHABET}{_VI_DIACRITIC}]+')

class ViRi55:

    vi_dict = RI55

    def __init__(self) -> None:
        pass

    def thap_kham(self, word: str) -> str:
        try:
            split_syl = SplitSyl(self.vi_dict)
            self.split = split_syl.split_syl(word)
        except NotVietnamese:
            return word

        self.shorten = False
        self.pref = {'clear_vowel': False, 'vowel_pair_form': {'อาย': 'อัย',
            'โอย': 'โอย'}}

        self.select_onset()
        self.select_vowel()
        self.select_coda()
        self.select_tone()
        self.tone_shorten()
        syl_info = {
            'onset': self.onset, 'vowel': self.vowel,
            'coda': self.coda, 'tone': self.tone
            }
        spell = SpellWord(**self.pref)
        result = spell.spell_out(**syl_info)
        return result

    def select_onset(self) -> None:
        self.onset = self.vi_dict['onset'][self.split[0]]
        self.check_gi(self.split[0], self.split[1])
        vowel_onset = self.vi_dict['vowel'][self.split[1]][0]
        if vowel_onset != 'อ':
            if self.split[0] == '':
                self.onset == vowel_onset
            else:
                self.onset = ''.join([self.onset, vowel_onset])

    def check_gi(self, original_onset: str, original_vowel: str) -> None:
        """Check for gi with (reduced) i as a vowel.
        
        gì should be สี่ not กี่"""
        if original_onset == 'g' and original_vowel == 'i':
            self.onset = self.vi_dict['onset']['gi']

    def select_vowel(self) -> None:
        self.vowel = self.vi_dict['vowel'][self.split[1]][1]
        if ((self.split[1] == 'a' and self.split[2] == 'nh') or
            (self.split[1] in ['ê', 'i', 'o', 'ô', 'u', 'ư', 'y'] and
            self.split[2] in ['c', 'ch', 'k', 'ng', 'nh'])):
            self.shorten = True

    def select_coda(self) -> None:
        if self.split[2] == '':
            self.coda = ''
        else:
            self.coda = self.vi_dict['coda'][self.split[2]]

    def select_tone(self) -> None:
        if self.split[3] == '':
            self.tone = ''
        else:
            self.tone = self.vi_dict['tone'][self.split[3]][0]
            if self.vi_dict['tone'][self.split[3]][1]:
                self.shorten = True

    def tone_shorten(self) -> None:
        if self.shorten == True:
            self.pref.update({'vowel_length': 'short'})

class ThapVi:
    def __init__(self, system: str = 'RI55', input: str = 'VA') -> None:
        """Setting
        
        :param system: Select thapsap system.
            - 'RI55' for the Royal Institute (2012/2555) system

        :param input: Select input type.
            - 'VA' for Vietnamese Alphabet
        """
        if system == 'RI55' and input == 'VA':
            self._transcript = ViRi55()
        else:
            raise NotAvailableSystem

    def thap(self, text: str) -> str:
        pattern = re.compile(_VI_ALL)
        def replace(match):
            return self._transcript.thap_kham(match.group())
        return re.sub(pattern, replace, text)

class NotAvailableSystem(Exception):
    pass


import unittest
from wunsen import ThapSap

GENERAL = {
    '日本語 にほんご Nihongo นิฮงโงะ': '日本語 にほんご นิฮงโงะ นิฮงโงะ',
    'ニホンゴ123456789': 'ニホンゴ123456789',
    """uzubekisutan
kazafusutan
kirugisu
tajikisutan
torukumenisutan

indoneshia
kanbojia
shingapōru
tai
higashitimōru
firipin
burunei
betonamu
marēshia
myanmā
raosu""":
"""อูซูเบกิซูตัง
คาซาฟูซูตัง
คิรูงิซุ
ทาจิกิซูตัง
โทรูกูเมนิซูตัง

อินโดเนชิอะ
คัมโบจิอะ
ชิงงาโปรุ
ไท
ฮิงาชิติโมรุ
ฟิริปิง
บูรูเน
เบโตนามุ
มาเรชิอะ
เมียมมา
ราโอซุ"""
}

ORS61_EXAMPLE = {
    "bon'odori": "บงโอโดริ",
    "obi": "โอบิ",
    "chīsai": "ชีไซ",
    "konnichiwa": "คนนิจิวะ",
    "denwa": "เด็งวะ",
    "Edo": "เอโดะ",
    "fune": "ฟูเนะ",
    "Gifu": "กิฟุ",
    "ginkō": "กิงโก",
    "arigatō": "อาริงาโต",
    "hashi": "ฮาชิ",
    "Hiroshima": "ฮิโรชิมะ",
    "Jōmon": "โจมง",
    "kaji": "คาจิ",
    "kao": "คาโอะ",
    "niku": "นิกุ",
    "gakkō": "กักโก",
    "mado": "มาโดะ",
    "shimbun": "ชิมบุง",
    "samma": "ซัมมะ",
    "empitsu": "เอ็มปิตสึ",
    "Nagoya": "นาโงยะ",
    "kinoko": "คิโนโกะ",
    "Nippon": "นิปปง",
    "konnichiwa": "คนนิจิวะ",
    "minchō": "มินโจ",
    "jinja": "จินจะ",
    "konnichiwa": "คนนิจิวะ",
    "konnyaku": "คนเนียกุ",
    "Endō": "เอ็นโด",
    "renraku": "เร็นรากุ",
    "hontō": "ฮนโต",
    "jinzai": "จินไซ",
    "ginnan": "กินนัง",
    "ringo": "ริงโงะ",
    "ginkō": "กิงโก",
    "kokusanhin": "โคกูซังฮิง",
    "denwa": "เด็งวะ",
    "hon'ya": "ฮงยะ", # honya => โฮเนีย
    "shinsai": "ชินไซ",
    "manshū": "มันชู",
    "bon'odori": "บงโอโดริ",
    "ichiban": "อิจิบัง",
    "pen": "เพ็ง",
    "tempura": "เท็มปูระ",
    "Nippon": "นิปปง",
    "renraku": "เร็นรากุ",
    "Nara": "นาระ",
    "sakana": "ซากานะ",
    "kissaten": "คิซซาเต็ง",
    "zasshi": "ซัชชิ",
    "Shōwa": "โชวะ",
    "sashimi": "ซาชิมิ",
    "te": "เทะ",
    "migite": "มิงิเตะ",
    "matcha": "มัตจะ",
    "kitte": "คิตเตะ",
    "tsunami": "สึนามิ",
    "mittsu": "มิตสึ",
    "mitsu": "มิตสึ",
    "tsūyaku": "ซือยากุ",
    "ittsū": "อิตซือ",
    "futsū": "ฟุตซือ",
    "watashi": "วาตาชิ",
    "Fujiwara": "ฟูจิวาระ",
    "yama": "ยามะ",
    "Yayoi": "ยาโยอิ",
    "zō": "โซ",
    "mizu": "มิซุ",

    "hyaku": "เฮียกุ",
    "kyakkan": "เคียกกัง",
    "kyā": "เคีย",
    "ryokō": "เรียวโก",
    "hyotto": "เฮียวโตะ",
    "ryōri": "เรียวริ",
    "byuffe": "บิวเฟะ",
    "kyūkō": "คีวโก",
    "Ryūkyū": "รีวกีว",

    "wasabi": "วาซาบิ",
    "yama": "ยามะ",
    "gakkō": "กักโก",
    "okāsan": "โอกาซัง",
    "haiku": "ไฮกุ",

    "eki": "เอกิ",
    "fune": "ฟูเนะ",
    "denwa": "เด็งวะ",
    "onēsan": "โอเนซัง",
    "sensei": "เซ็นเซ",

    "kaki": "คากิ",
    "kin": "คิง",
    "oishī": "โออิชี",

    "ocha": "โอจะ",
    "oto": "โอโตะ",
    "konnichiwa": "คนนิจิวะ",
    "sayōnara": "ซาโยนาระ",
    "Sōseki": "โซเซกิ",
    "Ōno": "โอโนะ",

    "Kabuki": "คาบูกิ",
    "isu": "อิซุ",
    "shimbun": "ชิมบุง",
    "jūyō": "จูโย",

    "Ichirō SUZUKI": "อิจิโร ซูซูกิ",
    "Takuya KIMURA": "ทากูยะ คิมูระ"
}

ORS61_EX_NO_DIACRITIC = {
    'okaasan': 'โอกาซัง',
    'oishii': 'โออิชี',
    'juuyoo': 'จูโย',
    'oneesan': 'โอเนซัง',
    'sensei': 'เซ็นเซ',
    'sayoonara': 'ซาโยนาระ',
    'koushi': 'โคชิ'
    # koushi (ko-ushi)
    # keito (ke-ito)
    # Ishii (Ishi-i)
}

RI35_EXAMPLE = {
    'obi': 'โอะบิ',
    'konbanwa': 'คมบังวะ',
    'chīsai': 'ชีซะอิ',
    'konnichiwa': 'คนนิชิวะ',
    'denwa': 'เด็งวะ',
    'Yamada': 'ยะมะดะ',
    'Fujisan': 'ฟุจิซัง',
    'fune': 'ฟุเนะ',
    'ginkō': 'กิงโก',
    'arigatō': 'อะริงะโต',
    'hashi': 'ฮะชิ',
    'Hiroshima': 'ฮิโระชิมะ',
    'kaji': 'คะจิ',
    'kao': 'คะโอะ',
    'niku': 'นิกุ',
    'gakkō': 'กักโก',
    'mado': 'มะโดะ',
    'Nagoya': 'นะโงะยะ',
    'konnichiwa': 'คนนิชิวะ',
    'pen': 'เพ็ง',
    'tenpura': 'เท็มปุระ',
    'Nippon': 'นิปปง',
    'ringo': 'ริงโงะ',
    'sakana': 'ซะกะนะ',
    'sashimi': 'ซะชิมิ',
    'kissaten': 'คิสซะเต็ง',
    'zasshi': 'ซัสชิ',
    'te': 'เทะ',
    'migite': 'มิงิเตะ',
    'itchi': 'อิตชิ',
    'tsukue': 'สึกุเอะ',
    'mittsu': 'มิตสึ',
    'watashi': 'วะตะชิ',
    'yama': 'ยะมะ',
    'mizu': 'มิซุ',

    'yama': 'ยะมะ',
    'sakura': 'ซะกุระ',
    'gakkō': 'กักโก',
    'san': 'ซัง',
    'okāsan': 'โอะกาซัง',
    'obāsan': 'โอะบาซัง',
    'ike': 'อิเกะ',
    'fune': 'ฟุเนะ',
    'denwa': 'เด็งวะ',
    'sensei': 'เซ็นเซ',
    'ē': 'เอ',
    'onēsan': 'โอะเนซัง',
    'sensei': 'เซ็นเซ',
    'kin': 'คิง',
    'kaki': 'คะกิ',
    'hashi': 'ฮะชิ',
    'onīsan': 'โอะนีซัง',
    'oishī': 'โอะอิชี',
    'ocha': 'โอะชะ',
    'kome': 'โคะเมะ',
    'Nippon': 'นิปปง',
    'konnichiwa': 'คนนิชิวะ',
    'otōsan': 'โอะโตซัง',
    'sayōnara': 'ซะโยนะระ',
    'shinbun': 'ชิมบุง',
    'isu': 'อิซุ',
    'Suzuki': 'ซุซุกิ',
    'jūyō': 'จูโย',
    'jūsho': 'จูโชะ',
    'kyaku': 'เคียะกุ',
    'hyaku': 'เฮียะกุ',
    'nyānyā': 'เนียเนีย',
    'ryokō': 'เรียวโก',
    'byōin': 'เบียวอิง',
    'ryōri': 'เรียวริ',
    'kyu': 'คิว',
    'kyūkō': 'คีวโก'
}

RI35_EX_NO_DIACRITIC = {
    'chiisai': 'ชีซะอิ',
    'ginkoo': 'กิงโก',
    'arigatoo': 'อะริงะโต',
    'gakkoo': 'กักโก',
    'okaasan': 'โอะกาซัง',
    'obaasan': 'โอะบาซัง',
    'ee': 'เอ',
    'oneesan': 'โอะเนซัง',
    'oniisan': 'โอะนีซัง',
    'oishii': 'โอะอิชี',
    'otoosan': 'โอะโตซัง',
    'sayoonara': 'ซะโยนะระ',
    'juuyoo': 'จูโย',
    'juusho': 'จูโชะ',
    'nyaanyaa': 'เนียเนีย',
    'ryokoo': 'เรียวโก',
    'byooin': 'เบียวอิง',
    'ryoori': 'เรียวริ',
    'kyuukoo': 'คีวโก'
}

class TestSpellWord(unittest.TestCase):

    def test_general(self):
        thap_sap = ThapSap('ja')
        for case in GENERAL:
            self.assertEqual(thap_sap.thap(case), GENERAL[case])

    def test_ors61_example(self):
        thap_sap = ThapSap('ja')
        for case in ORS61_EXAMPLE:
            self.assertEqual(thap_sap.thap(case), ORS61_EXAMPLE[case])

    def test_ors61_ex_no_diacritic(self):
        thap_sap = ThapSap('ja', input='Hepburn-no diacritic')
        for case in ORS61_EX_NO_DIACRITIC:
            self.assertEqual(thap_sap.thap(case), ORS61_EX_NO_DIACRITIC[case])

    def test_ri35_example(self):
        thap_sap = ThapSap('ja', system='RI35')
        for case in RI35_EXAMPLE:
            self.assertEqual(thap_sap.thap(case), RI35_EXAMPLE[case])

    def test_ri35_ex_no_diacritic(self):
        thap_sap = ThapSap('ja', system='RI35', input='Hepburn-no diacritic')
        for case in RI35_EX_NO_DIACRITIC:
            self.assertEqual(thap_sap.thap(case), RI35_EX_NO_DIACRITIC[case])

if __name__ == '__main__':
    unittest.main()


import re
from typing import Any, Dict, List, Pattern, Tuple

class NotInDict(Exception):
    pass

class SplitSyl:
    def __init__(self, transcription_dict: Dict[str, Any]):
        self.ts_dict = transcription_dict

    def split_syl(self, word: str) -> List[List[str]]:
        word = word.lower()
        group_split = SplitSyl.grouping(SplitSyl.split_vowel(self, word), 2)
        con_exp = self.find_con_exp()
        result = []

        for syl in group_split[:-1]:
            con_split = SplitSyl.split_con(self, con_exp, syl[0])
            con_split = SplitSyl.find_coda_onset(self, con_split)
            result.extend([con_split[0], con_split[1], syl[1]])

        SplitSyl.check_coda(self, group_split[-1][0])
        result.append(group_split[-1][0])
        SplitSyl.check_first(result)
        del result[0]
        return SplitSyl.grouping(result, 3)

    @staticmethod
    def grouping(list: List[str], group_length: int) -> List[List[str]]:
        return [list[i:i+group_length]
            for i in range(0, len(list), group_length)]

    def split_vowel(self, word: str) -> List[str]:
        vowel_pattern = '|'.join(sorted(
            self.ts_dict['vowel'],
            key=lambda x: len(x),
            reverse=True))
        vowel_pattern = f'({vowel_pattern})'

        expression = re.compile(vowel_pattern)

        return re.split(expression, word)

    def find_con_exp(self) -> Pattern[str]:
        all_onset = '$|'.join(sorted(self.ts_dict['onset'],
            key=lambda con: len(con), reverse=True))
        con_pattern = rf'({all_onset}$)'
        return re.compile(con_pattern)

    def split_con(self, con_pattern: Pattern[str], con: str) -> List[str]:
        if con == '' or con == "'":
            return ['', '']
        elif con.find("'") != -1:
            return SplitSyl.comma_case(self, con)
        else:
            con_split = re.split(con_pattern, con)
            con_split = [found for found in con_split if found]
            return con_split

    def comma_case(self, con: str) -> List[str]:
        parts = con.partition("'")
        if ((parts[0] == ''
                or parts[0] in self.ts_dict['coda'])
                and (parts[2] == ''
                or parts[2] in self.ts_dict['onset'])):
            return [parts[0], parts[2]]
        else:
            raise NotInDict

    def find_coda_onset(self, con_split: List[str]) -> Tuple[str, str]:
        if len(con_split) > 2:
            raise NotInDict
        elif len(con_split) == 2:
            if (con_split[0] != ''
                    and con_split[0] not in self.ts_dict['coda']
                    or (con_split[1] != ''
                    and con_split[1] not in self.ts_dict['onset'])):
                raise NotInDict
            coda = con_split[0]
            onset = con_split[-1]
        else:
            if (con_split[0] != ''
                    and con_split[0] not in self.ts_dict['onset']):
                if con_split[0] not in self.ts_dict['coda']:
                    raise NotInDict
                else:
                    coda = con_split[0]
                    onset = ''
            else:
                coda = ''
                onset = con_split[0]
        return coda, onset

    def check_coda(self, con: str) -> None:
        if con not in self.ts_dict['coda'] and con != '':
            raise NotInDict
        else:
            return

    def check_first(syl_list: List[str]) -> None:
        if syl_list[0] != '':
            raise NotInDict
        else:
            return